@@ -37,7 +37,7 @@ def post_process(graph, dc):
     :return: expanded graph
     '''
     for node in dc:
-        graph += '[' + node + ']'
+        graph[node] = {'par': [], 'nei': []}
     return graph
 
 
@@ -277,7 +277,6 @@ def hc(data, method='standard', score_function='default', debug=False):
                         dag[vi]['par'] = sorted(dag[vi]['par'] + [par_vi])
             dag, candidate = dag_update(dag, edge_candidate, candidate, diff, debug)
             cache_dag.append({var: {'par': [v for v in dag[var]['par']], 'nei': [v for v in dag[var]['nei']]} for var in dag})
-        dag = to_bnlearn(dag)
         time_total = time.time() - time_total
         if debug:
             print('total cost: %.2f seconds' % time_total)
@@ -371,7 +370,6 @@ def hc(data, method='standard', score_function='default', debug=False):
                     if debug:
                         print(par_vi, vi, diff_temp, 'd')
             dag, candidate = dag_update(dag, edge_candidate, candidate, diff, debug)
-        dag = to_bnlearn(dag)
         time_total = time.time() - time_total
         if debug:
             print('total cost:', time_total, 'seconds')
@@ -441,9 +439,9 @@ def pc_stable(data, test='default', threshold=0.05, debug=False):
     cpdag = orient_v_structure(data, varnames, pc, sepset, test, threshold=threshold)
     # orientation propagation
     cpdag = orient_propagation(cpdag)
-    dag = pdag2dag(cpdag)
-    dag = to_bnlearn(dag)
-    return post_process(dag, dc)
+    # force undirected edges to be directed if its orientation causes either cycles or new v-structures (force them to form v-structures)
+    cpdag = dag2cpdag(cpdag)
+    return post_process(cpdag, dc)
 
 
 # def h2pc(data, debug=False):
@@ -470,11 +468,7 @@ def ges(data, method='complete', score_function='default', prune=True, thres=0.1
     :param thres: threshold for CI test used in pruning process
     :return: the learned CPDAG
     '''
-    # remove unused states
-
-    dc = data.nunique() == 1
-    dc = dc[dc].index.values
-    data = data.drop(dc, axis=1)
+    data, dc = pre_process(data)
 
     # calculate the run time
     time_total = time.time()
@@ -485,7 +479,6 @@ def ges(data, method='complete', score_function='default', prune=True, thres=0.1
     if method == 'complete':
         if all(data[var].dtype.name == 'category' for var in data):
             datatype = 'category'
-            data = pandas.DataFrame(data.to_numpy(), columns=list(data.columns), dtype='category')
             data = data.apply(lambda x: x.cat.codes).to_numpy()
             if score_function == 'default':
                 score_function = 'bic'
@@ -747,12 +740,7 @@ def ges(data, method='complete', score_function='default', prune=True, thres=0.1
     if debug:
         print('total cost:', time_total, 'seconds')
         # print('compute score cost: %.2f%%' % (time_score / time_total * 100))
-
-    dag = to_bnlearn(pdag2dag(cpdag))
-    if len(dc):
-        for node in dc:
-            dag += '[' + node + ']'
-    return dag
+    return post_process(cpdag, dc)
 
 
 def all_subsets(ss):
